/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SMARTCAM_H_
#define _SMARTCAM_H_

#include <FApp.h>
#include <FBase.h>
#include <FSystem.h>
#include <FUi.h>

#include "comm/CommHandler.h"
#include "video/CameraHandler.h"
#include "ConnectionThread.h"

class CameraForm;
class SettingsForm;
class HelpForm;
class AboutForm;
class MainMenuForm;
class CameraHandler;
class JpegHandler;
class UserSettings;
class SettingsCamSrcDlgForm;

/**
 * [SmartCam] application must inherit from Application class
 * which provides basic features necessary to define an application.
 */
class SmartCam :
	public Osp::App::Application,
	public Osp::System::IScreenEventListener,
	public CommHandlerObserver,
	public CameraHandlerObserver
{
public:

	/**
	 * [SmartCam] application must have a factory method that creates an instance of itself.
	 */
	static Osp::App::Application* CreateInstance(void);
	enum { EVENT_ON_CONNECTED, EVENT_ON_CONNECTION_ERROR, EVENT_ON_DISCONNECTED };

public:
	SmartCam();
	~SmartCam();

	void dispose();

	void DisplayCameraForm();
	void DisplaySettingsForm();
	void DisplayHelpForm();
	void DisplayAboutForm();
	void DisplayMainMenuForm();

	void ShowErrorMsg(const String& errMsg);
	void ShowErrorMsgAndExit(const String& errMsg);

	void ShowForm(Osp::Ui::Controls::Form* pForm);

	UserSettings& GetSettings();
	void SaveSettings(UserSettings* pNewSettings);
	Osp::Graphics::Bitmap *bmpBGP;
	Osp::Graphics::Bitmap *bmpBGL;

public:
	// Called when the application is initializing.
	bool OnAppInitializing(Osp::App::AppRegistry& appRegistry);
	// Called when the application is terminating.
	bool OnAppTerminating(Osp::App::AppRegistry& appRegistry, bool forcedTermination = false);
	// Called when the application's frame moves to the top of the screen.
	void OnForeground();
	// Called when this application's frame is moved from top of the screen to the background.
	void OnBackground();
	// Called when the system memory is not sufficient to run the application any further.
	void OnLowMemory();
	// Called when the battery level changes.
	void OnBatteryLevelChanged(Osp::System::BatteryLevel batteryLevel);
	// Called when the screen turns on.
	void OnScreenOn();
	// Called when the screen turns off.
	void OnScreenOff();

	bool IsConnected();
	// From CommHandlerObserver:
	void OnConnected();
	void OnConnectError(result errCode, const String& errMsg);
	void OnDisconnected();

	// From CameraHandlerObserver
	void OnFrameAvailable(Osp::Base::ByteBuffer& frameData);

	Osp::Graphics::Dimension GetPreviewResolution();

	Osp::Base::Collection::IListT<Osp::Graphics::PixelFormat>* GetMainPreviewFormatList();
	Osp::Base::Collection::IList* GetMainPreviewResolutionList();
	Osp::Base::Collection::IListT<int>* GetMainPreviewFPSList();

	Osp::Base::Collection::IListT<Osp::Graphics::PixelFormat>* GetFrontPreviewFormatList();
	Osp::Base::Collection::IList* GetFrontPreviewResolutionList();
	Osp::Base::Collection::IListT<int>* GetFrontPreviewFPSList();

	bool IsMainCameraAvailable();
	bool IsFrontCameraAvailable();

	// New methods:
	void Connect();
	void Disconnect();
	void OnUserEventReceivedN(RequestId requestId, Osp::Base::Collection::IList* pArgs);


//private:
public:
	// Methods:
	// Utility for quick retrieval of localized strings
    Osp::Base::String GS(Osp::Base::String resId);
	// Data:
	CameraHandler* pCamera;
	bool isMainCameraAvailable;
	bool isFrontCameraAvailable;
	Osp::Base::Collection::IListT<Osp::Graphics::PixelFormat>* pMainPreviewFormatList;
	Osp::Base::Collection::IList* pMainPreviewResolutionList;
	Osp::Base::Collection::IListT<int>* pMainPreviewFPSList;
	Osp::Base::Collection::IListT<Osp::Graphics::PixelFormat>* pFrontPreviewFormatList;
	Osp::Base::Collection::IList* pFrontPreviewResolutionList;
	Osp::Base::Collection::IListT<int>* pFrontPreviewFPSList;

	CommHandler* pCommHandler;
	JpegHandler* pJpegHandler;
	UserSettings* pSettings;

	// Forms
	CameraForm* pCameraForm;
	SettingsForm* pSettingsForm;
	HelpForm* pHelpForm;
	AboutForm* pAboutForm;
	MainMenuForm *pMainMenuForm;
	// init error msg (if any) to be displayed on OnAppTerminating()
	String initErrMsg;
	bool isTerminating;
	class ConnectionThread *pConnThread;
};

#endif
