/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __COMM_HANDLER_H__
#define __COMM_HANDLER_H__

#include <FNet.h>

#include <list>

using namespace Osp::Base;
using namespace Osp::Net;
using namespace Osp::Net::Sockets;
using namespace std;

class CommHandlerObserver
{
public:
	virtual void OnConnected() = 0;
	virtual void OnConnectError(result errCode, const String& errMsg) = 0;
	virtual void OnDisconnected() = 0;
};

class CommHandler :
	public IDnsEventListener
{
public:
	typedef enum
	{
		Disconnected = 0,
		Connecting = 1,
		Connected = 2
	}CommState;

	CommHandler(CommHandlerObserver& commObserver);
	~CommHandler();
	result Construct();
	result Connect(String addr, int portNo);
	result Disconnect();
	result SendData(ByteBuffer* jpegData);
	CommState GetCommState();

public:
	result CreateSocket();
	// From IDnsEventListener:
	void OnDnsResolutionCompletedN (IpHostEntry* pIpHostEntry, result r);

private:
	// Utility for quick retrieval of localized strings
    Osp::Base::String GS(Osp::Base::String resId);
	// private methods:
	result ConnectIPAddr(const IpAddress& ipAddr);
	void SendNextInTxQ();
	// Data:
	Socket* pSocket;
	Dns* pDns;
	int port;
	CommState commState;
	CommHandlerObserver& observer;
};

#endif//__COMM_HANDLER_H__
