/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HELPFORM_H_
#define _HELPFORM_H_

#include <FBase.h>
#include <FUi.h>
#include <FWeb.h>

class SmartCam;

class HelpForm :
	public Osp::Ui::Controls::Form,
	public Osp::Ui::IActionEventListener,
	public Osp::Web::Controls::ILoadingListener,
	public Osp::Web::Controls::IWebDownloadListener,
	public Osp::Ui::IOrientationEventListener
{

// Construction
public:
	HelpForm(void);
	virtual ~HelpForm(void);
	bool Initialize();
	result OnInitializing(void);
	result OnTerminating(void);

// Implementation
protected:
	static const int ID_RIGHT_SOFTKEY = 900;

public:
	void OnActionPerformed(const Osp::Ui::Control&, int);
	// From Osp::Ui::IOrientationEventListener
	void OnOrientationChanged(const Osp::Ui::Control&, Osp::Ui::OrientationStatus);
	// From Osp::Web::Controls::ILoadingListener
	void OnEstimatedProgress(int progress);
	void OnHttpAuthenticationCanceled();
	bool OnHttpAuthenticationRequestedN(const Osp::Base::String& host, const Osp::Base::String& realm, const Osp::Web::Controls::AuthenticationChallenge& authentication);
	void OnLoadingCanceled();
	void OnLoadingCompleted();
	void OnLoadingErrorOccurred(Osp::Web::Controls::LoadingErrorType error, const Osp::Base::String& reason);
	bool OnLoadingRequested(const Osp::Base::String& url, Osp::Web::Controls::WebNavigationType type);
	void OnLoadingStarted();
	void OnPageTitleReceived(const Osp::Base::String& title);
	Osp::Web::Controls::DecisionPolicy OnWebDataReceived(const Osp::Base::String& mime, const Osp::Net::Http::HttpHeader& httpHeader);
	// From Osp::Web::Controls::IWebDownloadListener
	void OnWebChunkedDataReceived(const Osp::Base::ByteBuffer& chunk);
	void OnWebDataDownloadCompleted();
	void OnWebDownloadFailed(Osp::Web::Controls::LoadingErrorType error);

private:
	// New methods:
	void DoLayout();
	bool IsHelpUrl(const Osp::Base::String& url);
	result LoadHelpUrl(const Osp::Base::String& url);
	// Utility for quick retrieval of localized strings
    Osp::Base::String GS(Osp::Base::String resId);
	// Data:
	SmartCam* pSmartCamApp;
	Osp::Base::String crtUrl;
	Osp::Web::Controls::Web* pWeb;
	int retryCount;
};

#endif
