/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SETTINGSFORM_H_
#define _SETTINGSFORM_H_

#include <FBase.h>
#include <FUi.h>

class SmartCam;
class UserSettings;

class SettingsForm :
	public Osp::Ui::Controls::Form,
	public Osp::Ui::IItemEventListener,
	public Osp::Ui::IGroupedItemEventListener,
	public Osp::Ui::IActionEventListener,
	public Osp::Ui::IOrientationEventListener
{

// Construction
public:
	SettingsForm();
	virtual ~SettingsForm();
	bool Initialize();
	result OnInitializing();
	result OnTerminating();
	void PopulateFromSettings();

// Implementation
protected:
	static const int ID_FORMAT_STRING = 501;
	static const int ID_FORMAT_BITMAP = 502;
	static const int ID_LISTITEM_CAM_SRC = 503;
	static const int ID_LISTITEM_CAM_RES = 504;
	static const int ID_LISTITEM_CAM_FPS = 505;
	static const int ID_LISTITEM_CONN_ADDR = 506;
	static const int ID_LISTITEM_CONN_PORT = 507;
	static const int ID_BTN_OK_CAM_SRC = 508;
	static const int ID_BTN_OK_CAM_RES = 509;
	static const int ID_BTN_OK_CAM_FPS = 510;
	static const int ID_BTN_OK_SRV_ADDR = 511;
	static const int ID_BTN_OK_SRV_PORT = 512;
	static const int ID_LISTITEM_CAM_SRC_MAIN = 513;
	static const int ID_LISTITEM_CAM_SRC_FRONT = 514;

	static const int ID_RIGHT_SOFTKEY = 900;

// Generated call-back functions
public:

	// From Osp::Ui::IItemEventListener:
	virtual void OnItemStateChanged (const Osp::Ui::Control &source, int index, int itemId, Osp::Ui::ItemStatus status);
	// From Osp::Ui::IGroupedItemEventListener:
	virtual void OnItemStateChanged(const Osp::Ui::Control &source, int groupIndex, int itemIndex, int itemId, int elementId, Osp::Ui::ItemStatus status);
	virtual void OnItemStateChanged(const Osp::Ui::Control &source, int groupIndex, int itemIndex, int itemId, Osp::Ui::ItemStatus status);

	void OnActionPerformed(const Osp::Ui::Control&, int);
	void OnOrientationChanged(const Osp::Ui::Control&, Osp::Ui::OrientationStatus);

private:
	// Methods:
	void ComputeLayouts();
	void DoLayout();
	void CreateCamSrcDlg();
	void CreateCamResDlg();
	void CreateCamFPSDlg();
	void CreateSrvAddrDlg();
	void CreateSrvPortDlg();

	void ShowPopup(Osp::Ui::Controls::Popup* pPopup);
	void HidePopup(Osp::Ui::Controls::Popup* pPopup);

	void PopulateWithMainCameraRes();
	void PopulateWithMainCameraFps();
	void PopulateWithFrontCameraRes();
	void PopulateWithFrontCameraFps();
	static int GetSelectedIndex(Osp::Ui::Controls::List* pList);
	void GetSettings();
	// Utility for quick retrieval of localized strings
    Osp::Base::String GS(Osp::Base::String resId);

    // Data:
	SmartCam* pSmartCamApp;
	UserSettings* pUserSettings;
	Osp::Ui::Controls::CustomListItemFormat* pItemFormat;
	Osp::Ui::Controls::GroupedList* pGroupedList;
	Osp::Ui::Controls::CustomListItem* pItemCameraSource;
	Osp::Ui::Controls::CustomListItem* pItemCameraResolution;
	Osp::Ui::Controls::CustomListItem* pItemCameraFPS;
	Osp::Ui::Controls::CustomListItem* pItemConnectionServer;
	Osp::Ui::Controls::CustomListItem* pItemConnectionPort;

	Osp::Ui::Controls::Popup* pCamSrcDlg;
	Osp::Ui::Controls::List* pCamSrcList;
	Osp::Ui::Controls::Button* pCamSrcOkBtn;

	Osp::Ui::Controls::Popup* pCamFpsDlg;
	Osp::Ui::Controls::List* pCamFpsList;
	int mainCamFpsSelIdx;
	int frontCamFpsSelIdx;
	Osp::Ui::Controls::Button* pCamFpsOkBtn;

	Osp::Ui::Controls::Popup* pCamResDlg;
	Osp::Ui::Controls::List* pCamResList;
	int mainCamResSelIdx;
	int frontCamResSelIdx;
	Osp::Ui::Controls::Button* pCamResOkBtn;

	Osp::Ui::Controls::Popup* pSrvAddrDlg;
	Osp::Ui::Controls::EditField* pSrvAddrEditField;
	Osp::Ui::Controls::Button* pSrvAddrOkBtn;

	Osp::Ui::Controls::Popup* pSrvPortDlg;
	Osp::Ui::Controls::EditField* pSrvPortEditField;
	Osp::Ui::Controls::Button* pSrvPortOkBtn;

	// Layout data:
	int groupedListWidth;
	int groupedListHeight;

	int listItemHeight;
	int listItemTextSize;
	int listItemImgSize;
	int listItemImgHorizMargin;
	Osp::Base::String srvAddrStr;
	int srvPort;
};

#endif
