/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __SMARTCAM_SETTINGS_H__
#define __SMARTCAM_SETTINGS_H__

#include <FBase.h>
#include <FApp.h>

/**
 * Stores the application settings.
 */
class UserSettings
{
	friend class SmartCam;
public:
	UserSettings();
    UserSettings(const UserSettings& settings);
	~UserSettings();
	UserSettings& operator=(const UserSettings& settings);
	bool operator==(const UserSettings& rhs) const;
	bool operator!=(const UserSettings& rhs) const;

public:
	// Data:
	int cameraSource;
	int mainCameraResolutionIdx;
	int mainCameraFPSIdx;
	int frontCameraResolutionIdx;
	int frontCameraFPSIdx;
	Osp::Base::String serverAddr;
	int serverPort;

private:
	static void GetIntVal(Osp::App::AppRegistry& appRegistry, const Osp::Base::String& key, int& val);
	static void GetStringVal(Osp::App::AppRegistry& appRegistry, const Osp::Base::String& key, Osp::Base::String& val);
    static UserSettings* LoadSettings();
    static void SaveSettings(UserSettings* pSettings);
};

#endif//__SMARTCAM_SETTINGS_H__
