/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __CAMERA_HANDLER_H__
#define __CAMERA_HANDLER_H__

#include <FMedia.h>

class CameraHandlerObserver
{
public:
	virtual void OnFrameAvailable(Osp::Base::ByteBuffer& frameData) = 0;
};

class CameraHandler	:
	public Osp::Media::Camera,
	public Osp::Media::ICameraEventListener
{
public:
	CameraHandler(CameraHandlerObserver& cameraObserver, Osp::Media::CameraSelection camSel);
	virtual ~CameraHandler();

public:
	result StartPreview(const Osp::Graphics::BufferInfo* pBufferInfo);
	result StopPreview();
	result Initialize();
	result Terminate();
	// Inherited purely virtual functions from ICameraEventListener
	void OnCameraAutoFocused(bool completeCondition);
	void OnCameraPreviewed(Osp::Base::ByteBuffer& previewedData , result r);
	void OnCameraCaptured(Osp::Base::ByteBuffer& capturedData , result r);
	void OnCameraErrorOccurred(Osp::Media::CameraErrorReason r);
public:
	// Helper static methods:
	static Osp::Base::String PixelFormat2Str(Osp::Graphics::PixelFormat pixelFormat);
	static Osp::Graphics::Dimension ScaleProportionally(int srcWidth, int srcHeight, int boundingWidth, int boundingHeight);
private:
	// Data:
	CameraHandlerObserver& observer;
	Osp::Media::CameraSelection cameraSel;
};

#endif
