/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// JpegHandler.h

#ifndef __JPEG_HANDLER_H__
#define __JPEG_HANDLER_H__

#include <FBase.h>

#include <stdio.h>
#include <list>

extern "C" {
#include "jpeglib.h"
#include "jerror.h"
}
//#include <machine/setjmp.h>

using namespace std;

#define DEFAULT_JPEG_WRITE_BUFFER_SIZE 3000

#define RGB565_MASK_RED 0xF800
#define RGB565_MASK_GREEN 0x07E0
#define RGB565_MASK_BLUE 0x001F

class JpegHandler
{
public:
	JpegHandler();
	~JpegHandler();

	void InitRGB();
	void InitYUV();

	int SetQuality(int quality);

	Osp::Base::ByteBuffer* GetOutputBuffer();

//!private:
public:
// Methods:
	void StartEncodeThread();
	void StopEncodeThread();
	int EncodeYUV420SP(const unsigned char* srcBuff, int width, int height);
	int EncodeYUV420P(const Osp::Base::ByteBuffer& srcByteBuff, int width, int height);
	int EncodeYV12(const unsigned char* srcBuff, int width, int height);
    int EncodeRGB24(const unsigned char* srcBuff, int width, int height);
	int EncodeRGB565(const unsigned char* srcBuff, int width, int height);

	static void error_exit(j_common_ptr cinfo);
	static void output_message(j_common_ptr cinfo);

	static void init_destination(j_compress_ptr cinfo);
	static boolean empty_output_buffer(j_compress_ptr cinfo);
	static void term_destination(j_compress_ptr cinfo);

// Data:
	int planeHeight;

	JSAMPARRAY yPlane;
	JSAMPARRAY uPlane;
	JSAMPARRAY vPlane;

	bool cinfoCreated;
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	struct jpeg_destination_mgr destMgr;

	//jmp_buf returnPoint;
	char messageBuffer[JMSG_LENGTH_MAX + 1];

	Osp::Base::ByteBuffer* headerAndWriteByteBuffer;
	unsigned char* headerAndWriteBuffer;
	unsigned char* writeBuffer;
	int writeBufferSize;
	int written;
	bool isEncodeThreadAlive;

	list<unsigned char*> encodeQ;
};

#endif//__JPEG_HANDLER_H__
