/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ConnectionThread.h"

using namespace Osp::Base;
using namespace Osp::Base::Collection;
using namespace Osp::Base::Utility;
using namespace Osp::Base::Runtime;

ConnectionThread::ConnectionThread(class SmartCam *p)
{
     parent = p;
}

ConnectionThread::~ConnectionThread()
{
	AppLog("~ConnectionThread START");
	if(pCommHandler != NULL)
	{
		delete pCommHandler;
		pCommHandler = NULL;
	}
	AppLog("~ConnectionThread END");
}

bool ConnectionThread::Init()
{
	this->Construct(THREAD_TYPE_EVENT_DRIVEN);

	pCommHandler = new CommHandler(*this);
	result r = pCommHandler->Construct();
	if(IsFailed(r))
	{
		return false;
	}
	return true;
}

// From CommHandlerObserver:
void ConnectionThread::OnConnected()
{
	AppLog("ConnectionThread::OnConnected()");
	parent->SendUserEvent(SmartCam::EVENT_ON_CONNECTED, null);
}

void ConnectionThread::OnConnectError(result errCode, const String& errMsg)
{
	AppLog("ConnectionThread::OnConnected()");

	ArrayList* pArgs = new ArrayList();
	pArgs->Construct();
	Integer *pErrCode = new Integer(errCode);
	String *pErrMsg = new String(errMsg);

    pArgs->Add(*pErrCode);
    pArgs->Add(*pErrMsg);

	parent->SendUserEvent(SmartCam::EVENT_ON_CONNECTION_ERROR, pArgs);
}

void ConnectionThread::OnDisconnected()
{
	AppLog("ConnectionThread::OnDisconnected()");
	if (!parent->isTerminating) parent->SendUserEvent(SmartCam::EVENT_ON_DISCONNECTED, null);
}

void ConnectionThread::OnUserEventReceivedN(RequestId requestId, Osp::Base::Collection::IList* pArgs)
{
	AppLog("ConnectionThread received event %d", requestId);
	switch(requestId)
	{
		case ConnectionThread::OPERATION_CONNECT:
		{
			Thread::Sleep(500);
			pCommHandler->Connect(parent->pSettings->serverAddr, parent->pSettings->serverPort);
		}
		break;
		case ConnectionThread::OPERATION_DISCONNECT:
		{
			//Thread::Sleep(500);
			pCommHandler->Disconnect();
		}
		break;
		default:
		{
		}
		break;
	}
}


