/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FBase.h>
#include <FIo.h>
#include <FText.h>

#include "Util.h"

Osp::Base::ByteBuffer* Util::ReadFileN(const Osp::Base::String& fileName)
{
	Osp::Base::ByteBuffer* pBb = NULL;
	result r = E_SUCCESS;
	Osp::Io::FileAttributes attr;
	long long size = 0;
	Osp::Io::File file;

	// Get file attribute (size)
	r = Osp::Io::File::GetAttributes(fileName, attr);
	if(IsFailed(r))
	{
		AppLogException("Util::ReadFileN(): error getting file attributes for file %S: (%d) %s", fileName.GetPointer(), r, GetErrorMessage(r));
		goto CATCH;
	}
	size = attr.GetFileSize();
	// Open the file
	r = file.Construct(fileName, L"r");
	if(IsFailed(r))
	{
		AppLogException("Util::ReadFileN(): error opening file %S: (%d) %s", fileName.GetPointer(), r, GetErrorMessage(r));
		goto CATCH;
	}
	// Construct ByteBuffer
	pBb = new Osp::Base::ByteBuffer();
	pBb->Construct(size);
	// Read the file
	r = file.Read(*pBb);
	if(IsFailed(r))
	{
		AppLogException("Util::ReadFileN(): error reading file %S: (%d) %s", fileName.GetPointer(), r, GetErrorMessage(r));
		goto CATCH;
	}

	AppLog("Util::ReadFileN(): SUCCESS (file size=%lld; bb position=%d; bb limit=%d; bb capacity=%d", size, pBb->GetPosition(), pBb->GetLimit(), pBb->GetCapacity());

CATCH:
	return pBb;
}

Osp::Base::ByteBuffer* Util::ConvertStringToByteBufferN(const Osp::Base::String& input)
{
	Osp::Text::AsciiEncoding ascii;
	Osp::Base::ByteBuffer* pOutput = ascii.GetBytesN(input);
	int remain = pOutput->GetRemaining();
	pOutput->SetLimit(remain - 1);
	return pOutput;
}
