/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FApp.h>

#include "gui/CameraForm.h"
#include "SmartCam.h"
#include "video/CameraHandler.h"

using namespace Osp::Base;
using namespace Osp::Ui;
using namespace Osp::Ui::Controls;
using namespace Osp::Graphics;

#define PREVIEW_OVERLAY_MIN_PAD_HORIZONTAL 20
#define PREVIEW_OVERLAY_MIN_PAD_VERTICAL 20

CameraForm::CameraForm():
	pSmartCamApp(NULL),
	pOverlayPanel(NULL),
	showCamPreview(false),
	isTerminated(false)
{
}

CameraForm::~CameraForm()
{
	AppLog("~CameraForm START");
	isTerminated = true;
	if(pBufferInfo != NULL)
	{
		delete pBufferInfo;
		pBufferInfo = NULL;
	}
	AppLog("~CameraForm END");
}

bool CameraForm::Initialize()
{
	pSmartCamApp = static_cast<SmartCam*>(Osp::App::Application::GetInstance());

	// Construct an XML form
	Construct(L"IDF_CAMERAFORM");

	Button *pB1 = static_cast<Button *>(GetControl(L"IDC_BUTTON1"));
	pB1->SetActionId(ID_OPTIONMENU_DISCONNECT);
	pB1->AddActionEventListener(*this);

	return true;
}

result CameraForm::OnInitializing()
{
	result r = E_SUCCESS;
	isTerminated = false;
	pBufferInfo = new BufferInfo();

	return r;
}

Osp::Graphics::BufferInfo* CameraForm::GetBufferInfo()
{
	return pBufferInfo;
}

result CameraForm::OnTerminating()
{
	result r = E_SUCCESS;

	isTerminated = true;

	delete pBufferInfo;
	pBufferInfo = NULL;
	return r;
}

result CameraForm::CreateOverlayPanel()
{
	AppLog("CameraForm::CreateOverlayPanel()");
	result r = E_SUCCESS;

    Osp::Graphics::Dimension camPreviewRes = pSmartCamApp->GetPreviewResolution();
    Rectangle clr = GetClientAreaBounds();
    int boundingW = clr.width - 80;
    int boundingH = clr.height - 80;

    Osp::Graphics::Dimension previewWAndH = CameraHandler::ScaleProportionally(
    		camPreviewRes.width, camPreviewRes.height, boundingW, boundingH);
    Osp::Graphics::Rectangle previewRect;
    previewRect.x = (clr.width - previewWAndH.width)/2;
    previewRect.y = (clr.height - previewWAndH.height)/2;
    previewRect.width = previewWAndH.width;
    previewRect.height = previewWAndH.height;
    AppLog("previewRect: %d %d %d %d", previewRect.x, previewRect.y, previewRect.width, previewRect.height);

    pOverlayPanel = new OverlayPanel();
    r = pOverlayPanel->Construct(previewRect);
    if(IsFailed(r))
    {
        AppLog("CameraForm::CreateOverlayPanel(): OverlayPanel construction failed.");
        goto CATCH;
    }

CATCH:
    return r;
}

void CameraForm::OnActionPerformed(const Osp::Ui::Control& source, int actionId)
{
	AppLog("CameraForm::OnActionPerformed");
	switch(actionId)
	{
	// Menu items
	case ID_OPTIONMENU_DISCONNECT:
		{
		    pSmartCamApp->Disconnect();
			//pSmartCamApp->SendUserEvent(SmartCam::EVENT_DISCONNECT, null);
		}
		break;
	default:
		break;
	}
}

result CameraForm::EnableCameraPreview(Osp::Ui::Orientation orientation)
{
	AppLog("CameraForm::EnableCameraPreview()");
	SetOrientation(orientation);
	showCamPreview = true;
	result r = CreateOverlayPanel();
	if(IsFailed(r))
	{
		goto CATCH;
	}

	r = AddControl(*pOverlayPanel);
    if(IsFailed(r))
    {
        AppLogException("CameraForm::EnableCameraPreview(): Adding overlaypanel failed (%d): %s!", r, GetErrorMessage(r));
        goto CATCH;
    }
    RequestRedraw();
	r = pOverlayPanel->GetBackgroundBufferInfo(*pBufferInfo);
	if(IsFailed(r))
	{
		AppLogException("CameraForm::EnableCameraPreview(): Panel's GetBackgroundBufferInfo failed (%d): %s", r, GetErrorMessage(r));
		goto CATCH;
	}
CATCH:
	if(IsFailed(r))
	{
		showCamPreview = false;
	}
	return r;
}

void CameraForm::DisableCameraPreview()
{
	AppLog("CameraForm::DisableCameraPreview()");
	if(pOverlayPanel != NULL && !isTerminated)
	{
		showCamPreview = false;
		//SetOrientation(Osp::Ui::ORIENTATION_AUTOMATIC);
		result r = RemoveControl(*pOverlayPanel);
		if(IsFailed(r))
		{
			AppLogException("CameraForm::DisableCameraPreview(): removing OverlayPanel failed.");
		}
		pOverlayPanel = NULL;
		RequestRedraw();
	}
}
