/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gui/HelpForm.h"
#include "gui/CameraForm.h"
#include "SmartCam.h"
#include "Util.h"

using namespace Osp::Base;
using namespace Osp::Ui;
using namespace Osp::Ui::Controls;


HelpForm::HelpForm(void):
	pSmartCamApp(NULL),
	crtUrl(""),
	pWeb(NULL),
	retryCount(0)
{
}

HelpForm::~HelpForm(void)
{
	AppLog("~HelpForm START");
	AppLog("~HelpForm END");
}

bool HelpForm::Initialize()
{
	pSmartCamApp = static_cast<SmartCam*>(Osp::App::Application::GetInstance());

	Form::Construct(L"IDF_HELPFORM");

	SetSoftkeyActionId(SOFTKEY_1, ID_RIGHT_SOFTKEY);
	AddSoftkeyActionListener(SOFTKEY_1, *this);

	return true;
}

result HelpForm::OnInitializing(void)
{
	result r = E_SUCCESS;
    Osp::Graphics::Rectangle rec = GetBounds();
	//SetOrientation(ORIENTATION_AUTOMATIC);
	AddOrientationEventListener(*this);

	DoLayout();

    pWeb = new Osp::Web::Controls::Web();
    pWeb->Construct(rec);
    pWeb->SetLoadingListener(this);
    pWeb->SetDownloadListener(this);
    AddControl(*pWeb);
    pWeb->SetFocus();

    LoadHelpUrl(GS("IDS_HELP_HTML"));
	return r;
}

bool HelpForm::IsHelpUrl(const Osp::Base::String& url)
{
	if(url.Equals(GS(L"IDS_HELP_CONNECT")) || url.Equals(GS(L"IDS_HELP_HTML")) || url.Equals(GS(L"IDS_HELP_INSTALL_PC")) ||
	   url.Equals(GS(L"IDS_HELP_SETTINGS")) || url.Equals(GS(L"IDS_HELP_WHATSNEW")) || url.Equals(GS(L"IDS_HELP_SUPPORT")))
		return true;
	return false;
}

result HelpForm::LoadHelpUrl(const Osp::Base::String& url)
{
	result r = E_FAILURE;
	if(IsHelpUrl(url))
	{
		crtUrl = url;
		Osp::Base::ByteBuffer* pBb = Util::ReadFileN(url);
		pWeb->LoadData(NULL, *pBb, "text/html", "UTF-8");
		delete pBb;
		r = E_SUCCESS;
	}
	return r;
}


result HelpForm::OnTerminating(void)
{
	result r = E_SUCCESS;
	return r;
}

void HelpForm::DoLayout()
{
	OrientationStatus orientationStatus = GetOrientationStatus();

	AppLog("HelpForm::DoLayout(): orientation = %d", orientationStatus);

	Osp::Graphics::Rectangle boundsRect;
	boundsRect = GetBounds();
	if(pWeb != NULL)
		pWeb->SetBounds(boundsRect);
}

void HelpForm::OnOrientationChanged(const Osp::Ui::Control&, Osp::Ui::OrientationStatus)
{
	DoLayout();
	RequestRedraw();
}

void HelpForm::OnActionPerformed(const Control& source, int actionId)
{
	switch(actionId)
	{
	// Soft keys:
	case ID_RIGHT_SOFTKEY:
		{
		if(IsHelpUrl(crtUrl))
		{
			if(crtUrl.Equals(GS(L"IDS_HELP_HTML")))
			{
				pSmartCamApp->DisplayMainMenuForm();
			}
			else
			{
				LoadHelpUrl(GS(L"IDS_HELP_HTML"));
			}
		}
		else
		{
			pWeb->GoBack();
		}
		}
		break;

	default:
		break;
	}
}

// From Osp::Web::Controls::ILoadingListener
void HelpForm::OnEstimatedProgress(int progress)
{
	AppLog("HelpForm::OnEstimatedProgress(): progress = %d", progress);
}

void HelpForm::OnHttpAuthenticationCanceled()
{
	AppLog("HelpForm::OnHttpAuthenticationCanceled()");
}

bool HelpForm::OnHttpAuthenticationRequestedN(const Osp::Base::String& host, const Osp::Base::String& realm, const Osp::Web::Controls::AuthenticationChallenge& authentication)
{
	AppLog("HelpForm::OnHttpAuthenticationRequestedN(): host=%S;realm=%S", host.GetPointer(), realm.GetPointer());
	return false;
}

void HelpForm::OnLoadingCanceled()
{
	AppLog("HelpForm::OnLoadingCanceled()");
}

void HelpForm::OnLoadingCompleted()
{
	AppLog("HelpForm::OnLoadingCompleted()");
}

void HelpForm::OnLoadingErrorOccurred(Osp::Web::Controls::LoadingErrorType error, const Osp::Base::String& reason)
{
	AppLog("HelpForm::OnLoadingErrorOccurred(): error=%d; reason=%S", (int)error, reason.GetPointer());
}

bool HelpForm::OnLoadingRequested(const Osp::Base::String& url, Osp::Web::Controls::WebNavigationType type)
{
	LoadHelpUrl(url);
	return false;
}

void HelpForm::OnLoadingStarted()
{
	AppLog("HelpForm::OnLoadingStarted()");
}

void HelpForm::OnPageTitleReceived(const Osp::Base::String& title)
{
	AppLog("HelpForm::OnPageTitleReceived(): title=%S", title.GetPointer());
	SetTitleText(title);
	RequestRedraw(true);
}

Osp::Web::Controls::DecisionPolicy HelpForm::OnWebDataReceived(const Osp::Base::String& mime, const Osp::Net::Http::HttpHeader& httpHeader)
{
	AppLog("HelpForm::OnWebDataReceived(): mime=%S", mime.GetPointer());
	return Osp::Web::Controls::WEB_DECISION_DOWNLOAD;
}

// From Osp::Web::Controls::IWebDownloadListener
void HelpForm::OnWebChunkedDataReceived(const Osp::Base::ByteBuffer& chunk)
{
	AppLog("HelpForm::OnWebChunkedDataReceived(): chunk limit=%d", chunk.GetLimit());
}

void HelpForm::OnWebDataDownloadCompleted()
{
	AppLog("HelpForm::OnWebDataDownloadCompleted()");
}

void HelpForm::OnWebDownloadFailed(Osp::Web::Controls::LoadingErrorType error)
{
	AppLog("HelpForm::OnWebDownloadFailed(): error=%d", (int)error);
}

String HelpForm::GS(String resId)
{
	String trans;
	SmartCam *pApp = static_cast<SmartCam*>(Osp::App::Application::GetInstance());
	pApp->GetAppResource()->GetString(resId, trans);
	if (trans.IsEmpty()) trans = resId;
	return trans;
}
