/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FApp.h>

#include "gui/MainMenuForm.h"
#include "SmartCam.h"
#include "video/CameraHandler.h"

using namespace Osp::Base;
using namespace Osp::Ui;
using namespace Osp::Ui::Controls;
using namespace Osp::Graphics;

#define PREVIEW_OVERLAY_MIN_PAD_HORIZONTAL 20
#define PREVIEW_OVERLAY_MIN_PAD_VERTICAL 20

MainMenuForm::MainMenuForm():
	pSmartCamApp(NULL),
	showCamPreview(false),
	isTerminated(false),
	pWaitDlg(NULL),
	pWaitAnimation(NULL),
	pAnimationFrameList(NULL)
{
}

MainMenuForm::~MainMenuForm()
{
	AppLog("~MainMenuForm START");
	isTerminated = true;
	AppLog("~MainMenuForm END");
}

bool MainMenuForm::Initialize()
{
	pSmartCamApp = static_cast<SmartCam*>(Osp::App::Application::GetInstance());

	// Construct an XML form
	Construct(L"IDF_MAINMENUFORM");

	Button *pB1 = static_cast<Button *>(GetControl(L"IDC_BUTTON1"));
	Button *pB2 = static_cast<Button *>(GetControl(L"IDC_BUTTON2"));
	Button *pB3 = static_cast<Button *>(GetControl(L"IDC_BUTTON3"));
	Button *pB4 = static_cast<Button *>(GetControl(L"IDC_BUTTON4"));
	Button *pB5 = static_cast<Button *>(GetControl(L"IDC_BUTTON5"));

	pB1->SetActionId(ID_OPTIONMENU_CONNECT);
	pB2->SetActionId(ID_OPTIONMENU_SETTINGS);
	pB3->SetActionId(ID_OPTIONMENU_HELP);
	pB4->SetActionId(ID_OPTIONMENU_ABOUT);
	pB5->SetActionId(ID_OPTIONMENU_EXIT);

	pB1->AddActionEventListener(*this);
	pB2->AddActionEventListener(*this);
	pB3->AddActionEventListener(*this);
	pB4->AddActionEventListener(*this);
	pB5->AddActionEventListener(*this);

	AddOrientationEventListener(*this);

	//SetOrientation(ORIENTATION_LANDSCAPE);
	//OnOrientationChanged(*this, GetOrientationStatus());

	return true;
}

result MainMenuForm::OnInitializing()
{
	result r = E_SUCCESS;
	isTerminated = false;

	CreateWaitDlg();

	return r;
}

result MainMenuForm::OnTerminating()
{
	result r = E_SUCCESS;

	isTerminated = true;

	if(pAnimationFrameList != NULL)
	{
		pAnimationFrameList->RemoveAll(true);
		delete pAnimationFrameList;
		pAnimationFrameList = NULL;
	}
	if(pWaitDlg != NULL)
	{
		delete pWaitDlg;
		pWaitDlg = NULL;
	}

	return r;
}

void MainMenuForm::OnActionPerformed(const Osp::Ui::Control& source, int actionId)
{
	switch(actionId)
	{
	// Menu items
	case ID_OPTIONMENU_CONNECT:
		{
		pSmartCamApp->Connect();
		}
		break;
	case ID_OPTIONMENU_SETTINGS:
		{
		pSmartCamApp->DisplaySettingsForm();
		}
		break;
	case ID_OPTIONMENU_HELP:
		{
		pSmartCamApp->DisplayHelpForm();
		}
		break;
	case ID_OPTIONMENU_ABOUT:
		{
		pSmartCamApp->DisplayAboutForm();
		}
		break;
	case ID_OPTIONMENU_EXIT:
		{
		pSmartCamApp->Terminate();
		}
		break;
	default:
		break;
	}
}

void MainMenuForm::CreateWaitDlg()
{
	AppLog("MainMenuForm::CreateWaitDlg()");
	pWaitDlg = new Popup();
	pWaitDlg->Construct(L"IDP_WAIT_DLG");
	Osp::Ui::Controls::Label* pWaitLabel = static_cast<Osp::Ui::Controls::Label*>(pWaitDlg->GetControl(L"IDC_LABEL"));
	pWaitLabel->SetBackgroundColor(Osp::Ui::Controls::SYSTEM_COLOR_POPUP_BACKGROUND);
	Osp::Ui::Controls::Panel* pAnimPanel = static_cast<Osp::Ui::Controls::Panel*>(pWaitDlg->GetControl(L"IDC_PANEL"));
	pAnimPanel->SetBackgroundColor(Osp::Ui::Controls::SYSTEM_COLOR_POPUP_BACKGROUND);
	CreateWaitAnimation();
	pAnimPanel->AddControl(*pWaitAnimation);
}

void MainMenuForm::CreateWaitAnimation()
{
	// Create Bitmap
	Osp::App::AppResource* pAppResource = pSmartCamApp->GetAppResource();
	Osp::Graphics::Bitmap* pBitmap1 = pAppResource->GetBitmapN("/progressing00_big.png");
	Osp::Graphics::Bitmap* pBitmap2 = pAppResource->GetBitmapN("/progressing01_big.png");
	Osp::Graphics::Bitmap* pBitmap3 = pAppResource->GetBitmapN("/progressing02_big.png");
	Osp::Graphics::Bitmap* pBitmap4 = pAppResource->GetBitmapN("/progressing03_big.png");
	Osp::Graphics::Bitmap* pBitmap5 = pAppResource->GetBitmapN("/progressing04_big.png");
	Osp::Graphics::Bitmap* pBitmap6 = pAppResource->GetBitmapN("/progressing05_big.png");
	Osp::Graphics::Bitmap* pBitmap7 = pAppResource->GetBitmapN("/progressing06_big.png");
	Osp::Graphics::Bitmap* pBitmap8 = pAppResource->GetBitmapN("/progressing07_big.png");

	// Create AnimationFrames
	long duration = 500 / 8;
	Osp::Ui::Controls::AnimationFrame* pAniFrame1 = new Osp::Ui::Controls::AnimationFrame(*pBitmap1, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame2 = new Osp::Ui::Controls::AnimationFrame(*pBitmap2, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame3 = new Osp::Ui::Controls::AnimationFrame(*pBitmap3, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame4 = new Osp::Ui::Controls::AnimationFrame(*pBitmap4, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame5 = new Osp::Ui::Controls::AnimationFrame(*pBitmap5, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame6 = new Osp::Ui::Controls::AnimationFrame(*pBitmap6, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame7 = new Osp::Ui::Controls::AnimationFrame(*pBitmap7, duration);
	Osp::Ui::Controls::AnimationFrame* pAniFrame8 = new Osp::Ui::Controls::AnimationFrame(*pBitmap8, duration);

	pAnimationFrameList = new Osp::Base::Collection::ArrayList();
	pAnimationFrameList->Construct();
	pAnimationFrameList->Add(*pAniFrame1);
	pAnimationFrameList->Add(*pAniFrame2);
	pAnimationFrameList->Add(*pAniFrame3);
	pAnimationFrameList->Add(*pAniFrame4);
	pAnimationFrameList->Add(*pAniFrame5);
	pAnimationFrameList->Add(*pAniFrame6);
	pAnimationFrameList->Add(*pAniFrame7);
	pAnimationFrameList->Add(*pAniFrame8);

	// Deallocate a Bitmap.
	delete pBitmap1;
	delete pBitmap2;
	delete pBitmap3;
	delete pBitmap4;
	delete pBitmap5;
	delete pBitmap6;
	delete pBitmap7;
	delete pBitmap8;

	pWaitAnimation = new Osp::Ui::Controls::Animation();
	pWaitAnimation->Construct(Osp::Graphics::Rectangle(0,0,60,60), *pAnimationFrameList);
	pWaitAnimation->SetRepeatCount(100000);
	pWaitAnimation->AddAnimationEventListener(*this);
}

void MainMenuForm::OnAnimationStopped(const Osp::Ui::Control& source)
{
	AppLog("MainMenuForm::OnAnimationStopped()");
}

void MainMenuForm::ShowWaitDlg()
{
	pWaitAnimation->Play();
	pWaitDlg->SetShowState(true);
	pWaitDlg->Show();
}

void MainMenuForm::HideWaitDlg()
{
	pWaitAnimation->Stop();
	pWaitDlg->SetShowState(false);
	Draw();
	Show();
}

void MainMenuForm::OnOrientationChanged(const Osp::Ui::Control &source, Osp::Ui::OrientationStatus orientationStatus)
{
	AppLog("Orientation changed!");
	if (!IsVisible()) return;
 	Label *pBg = static_cast<Label *>(GetControl(L"IDC_LABEL1"));

	if (orientationStatus == ORIENTATION_STATUS_PORTRAIT ||
			orientationStatus == ORIENTATION_STATUS_PORTRAIT_REVERSE)
	{
		pBg->SetBackgroundBitmap(*(pSmartCamApp->bmpBGP));
		pBg->SetBounds(0,0,480,762);
	}
	else if (orientationStatus == ORIENTATION_STATUS_LANDSCAPE ||
			orientationStatus == ORIENTATION_STATUS_LANDSCAPE_REVERSE)
	{
		pBg->SetBackgroundBitmap(*(pSmartCamApp->bmpBGL));
		pBg->SetBounds(0,0,800,442);
	}
	Draw();
	Show();
}
