/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gui/SettingsForm.h"
#include "settings/UserSettings.h"
#include "SmartCam.h"

#include <FMedia.h>

using namespace Osp::Base;
using namespace Osp::Ui;
using namespace Osp::Ui::Controls;
using namespace Osp::Graphics;

// Grouped list
#define GROUPED_LIST_WIDTH_PORTRAIT 480
#define GROUPED_LIST_HEIGHT_PORTRAIT 800

#define GROUPED_LIST_WIDTH_LANDSCAPE 800
#define GROUPED_LIST_HEIGHT_LANDSCAPE 480

// List item
#define LIST_ITEM_HEIGHT_PORTRAIT 40
#define LIST_ITEM_TEXT_SIZE_PORTRAIT 32
#define LIST_ITEM_IMG_SIZE_PORTRAIT 44
#define LIST_ITEM_IMG_HORIZ_MARGIN_PORTRAIT 4

#define LIST_ITEM_HEIGHT_LANDSCAPE 40
#define LIST_ITEM_TEXT_SIZE_LANDSCAPE 32
#define LIST_ITEM_IMG_SIZE_LANDSCAPE 44
#define LIST_ITEM_IMG_HORIZ_MARGIN_LANDSCAPE 4

SettingsForm::SettingsForm():
	pSmartCamApp(NULL),
	pUserSettings(NULL),
	pItemFormat(NULL),
	pGroupedList(NULL),
	pItemCameraSource(NULL),
	pItemCameraResolution(NULL),
	pItemCameraFPS(NULL),
	pItemConnectionServer(NULL),
	pItemConnectionPort(NULL),
	pCamSrcDlg(NULL),
	pCamSrcList(NULL),
	pCamSrcOkBtn(NULL),
	pCamFpsDlg(NULL),
	pCamFpsList(NULL),
	mainCamFpsSelIdx(0),
	frontCamFpsSelIdx(0),
	pCamFpsOkBtn(NULL),
	pCamResDlg(NULL),
	pCamResList(NULL),
	mainCamResSelIdx(0),
	frontCamResSelIdx(0),
	pCamResOkBtn(NULL),
	pSrvAddrDlg(NULL),
	pSrvAddrEditField(NULL),
	pSrvAddrOkBtn(NULL),
	pSrvPortDlg(NULL),
	pSrvPortEditField(NULL),
	pSrvPortOkBtn(NULL),
	groupedListWidth(0),
	groupedListHeight(0),
	listItemHeight(0),
	listItemTextSize(0),
	listItemImgSize(0),
	listItemImgHorizMargin(0)
{
}

SettingsForm::~SettingsForm()
{
	AppLog("~SettingsForm START");
	AppLog("~SettingsForm END");
}

bool SettingsForm::Initialize()
{
	pUserSettings = new UserSettings();
	pSmartCamApp = static_cast<SmartCam*>(Osp::App::Application::GetInstance());

	Construct(FORM_STYLE_NORMAL | FORM_STYLE_TITLE | FORM_STYLE_INDICATOR | FORM_STYLE_SOFTKEY_1);
	SetTitleText(GS(L"IDS_SETTINGS"));

	SetSoftkeyText(SOFTKEY_1, GS(L"IDS_OK"));
	SetSoftkeyActionId(SOFTKEY_1, ID_RIGHT_SOFTKEY);
	AddSoftkeyActionListener(SOFTKEY_1, *this);

	return true;
}

result SettingsForm::OnInitializing()
{
	result r = E_SUCCESS;
	SetOrientation(ORIENTATION_LANDSCAPE);
	//AddOrientationEventListener(*this);

	DoLayout();

	CreateCamSrcDlg();
	CreateCamResDlg();
	CreateCamFPSDlg();
	CreateSrvAddrDlg();
	CreateSrvPortDlg();

	// Camera
	pItemCameraSource = new CustomListItem();
	pItemCameraSource->Construct(listItemHeight);
	pItemCameraSource->SetItemFormat(*pItemFormat);
	pItemCameraSource->SetElement(ID_FORMAT_STRING, GS(L"IDS_SOURCE:"));

	pItemCameraResolution = new CustomListItem();
	pItemCameraResolution->Construct(listItemHeight);
	pItemCameraResolution->SetItemFormat(*pItemFormat);
	pItemCameraResolution->SetElement(ID_FORMAT_STRING, GS(L"IDS_RESOLUTION:"));

	pItemCameraFPS = new CustomListItem();
	pItemCameraFPS->Construct(listItemHeight);
	pItemCameraFPS->SetItemFormat(*pItemFormat);
	pItemCameraFPS->SetElement(ID_FORMAT_STRING, GS(L"IDS_FRAME_RATE:"));

	// Connection
	pItemConnectionServer = new CustomListItem();
	pItemConnectionServer->Construct(listItemHeight);
	pItemConnectionServer->SetItemFormat(*pItemFormat);
	pItemConnectionServer->SetElement(ID_FORMAT_STRING, GS(L"IDS_SERVER_ADDRESS:"));

	pItemConnectionPort = new CustomListItem();
	pItemConnectionPort->Construct(listItemHeight);
	pItemConnectionPort->SetItemFormat(*pItemFormat);
	pItemConnectionPort->SetElement(ID_FORMAT_STRING, GS(L"IDS_PORT"));

	pGroupedList = new GroupedList();
	pGroupedList->Construct(Rectangle(0, 0, groupedListWidth, groupedListHeight), CUSTOM_LIST_STYLE_NORMAL, true, true);
	pGroupedList->AddGroupedItemEventListener(*this);

	pGroupedList->AddGroup(GS(L"IDS_CAMERA"), NULL);
	pGroupedList->AddItem(0, *pItemCameraSource, ID_LISTITEM_CAM_SRC);
	pGroupedList->AddItem(0, *pItemCameraResolution, ID_LISTITEM_CAM_RES);
	pGroupedList->AddItem(0, *pItemCameraFPS, ID_LISTITEM_CAM_FPS);
	pGroupedList->AddGroup(GS(L"IDS_CONNECTION"),null);
	pGroupedList->AddItem(1, *pItemConnectionServer, ID_LISTITEM_CONN_ADDR);
	pGroupedList->AddItem(1, *pItemConnectionPort, ID_LISTITEM_CONN_PORT);

	AddControl(*pGroupedList);

	return r;
}

result SettingsForm::OnTerminating()
{
	result r = E_SUCCESS;

	delete pItemFormat;

	delete pCamSrcDlg;
	delete pCamFpsDlg;
	delete pCamResDlg;
	delete pSrvAddrDlg;
	delete pSrvPortDlg;

	delete pUserSettings;

	return r;
}

void SettingsForm::PopulateFromSettings()
{
	AppLog("SettingsForm::PopulateFromSettings()");
	UserSettings settings = pSmartCamApp->GetSettings();
	pCamSrcList->SetItemEnabled(0, pSmartCamApp->IsMainCameraAvailable());
	pCamSrcList->SetItemEnabled(1, pSmartCamApp->IsFrontCameraAvailable());

	if(settings.cameraSource == Osp::Media::CAMERA_PRIMARY && !pSmartCamApp->IsMainCameraAvailable())
	{
		settings.cameraSource = Osp::Media::CAMERA_SECONDARY;
	}
	else if(settings.cameraSource == Osp::Media::CAMERA_SECONDARY && !pSmartCamApp->IsFrontCameraAvailable())
	{
		settings.cameraSource = Osp::Media::CAMERA_PRIMARY;
	}

	pCamSrcList->SetItemChecked(0, settings.cameraSource == Osp::Media::CAMERA_PRIMARY);
	pCamSrcList->SetItemChecked(1, settings.cameraSource == Osp::Media::CAMERA_SECONDARY);
	if(pCamSrcList->IsItemChecked(0))// MAIN camera
	{
		// Camera source
		pItemCameraSource->SetElement(ID_FORMAT_STRING, GS(L"IDS_SOURCE:") + "  " + GS(L"IDS_MAIN_CAMERA"));
		pGroupedList->RefreshItem(0, 0);
		// Camera resolution
		Dimension* pCrtPreviewResolution = (Dimension*)pSmartCamApp->GetMainPreviewResolutionList()->GetAt(settings.mainCameraResolutionIdx);
		String crtResolutionStr = GS(L"IDS_RESOLUTION:") + "  " + Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
		pItemCameraResolution->SetElement(ID_FORMAT_STRING, crtResolutionStr);
		pGroupedList->RefreshItem(0, 1);
		// Camera FPS
		int crtFPS = 0;
		pSmartCamApp->GetMainPreviewFPSList()->GetAt(settings.mainCameraFPSIdx, crtFPS);
		String crtFPSStr = GS(L"IDS_FRAME_RATE:") + "  " + Integer::ToString(crtFPS) + String(" fps");
		pItemCameraFPS->SetElement(ID_FORMAT_STRING, crtFPSStr);
		pGroupedList->RefreshItem(0, 2);
	}
	else if(pCamSrcList->IsItemChecked(1)) // FRONT camera
	{
		// Camera source
		pItemCameraSource->SetElement(ID_FORMAT_STRING, GS(L"IDS_SOURCE:") + "  " + GS(L"IDS_FRONT_CAMERA"));
		pGroupedList->RefreshItem(0, 0);
		// Camera resolution
		Dimension* pCrtPreviewResolution = (Dimension*)pSmartCamApp->GetFrontPreviewResolutionList()->GetAt(settings.frontCameraResolutionIdx);
		String crtResolutionStr =  GS(L"IDS_RESOLUTION:") + "  " + Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
		pItemCameraResolution->SetElement(ID_FORMAT_STRING, crtResolutionStr);
		pGroupedList->RefreshItem(0, 1);
		// Camera FPS
		int crtFPS = 0;
		pSmartCamApp->GetFrontPreviewFPSList()->GetAt(settings.frontCameraFPSIdx, crtFPS);
		String crtFPSStr = GS(L"IDS_FRAME_RATE:") + "  " + Integer::ToString(crtFPS) + String(" fps");
		pItemCameraFPS->SetElement(ID_FORMAT_STRING, crtFPSStr);
		pGroupedList->RefreshItem(0, 2);
	}
	mainCamResSelIdx = settings.mainCameraResolutionIdx;
	mainCamFpsSelIdx = settings.mainCameraFPSIdx;
	frontCamResSelIdx = settings.frontCameraResolutionIdx;
	frontCamFpsSelIdx = settings.frontCameraFPSIdx;
	if(pCamSrcList->IsItemChecked(0)) // MAIN camera selected in settings
	{
		PopulateWithMainCameraRes();
		PopulateWithMainCameraFps();
	}
	else if(pCamSrcList->IsItemChecked(1)) // FRONT camera selected in setttings
	{
		PopulateWithFrontCameraRes();
		PopulateWithFrontCameraFps();
	}
	pItemConnectionServer->SetElement(ID_FORMAT_STRING, GS(L"IDS_SERVER_ADDRESS:") + "  " + settings.serverAddr);
	pGroupedList->RefreshItem(1, 0);
	settings.serverAddr.Trim();
	srvAddrStr = settings.serverAddr;
	pSrvAddrEditField->SetText(srvAddrStr);
	String srvPortStr = GS(L"IDS_PORT:") + "  " + Integer::ToString(settings.serverPort);
	pItemConnectionPort->SetElement(ID_FORMAT_STRING, srvPortStr);
	pGroupedList->RefreshItem(1, 1);
	srvPort = settings.serverPort;
	pSrvPortEditField->SetText(Integer::ToString(srvPort));

}

void SettingsForm::GetSettings()
{
	AppLog("SettingsForm::GetSettings()");
	// cameraSource
	if(pCamSrcList->IsItemChecked(0))// MAIN camera selected
	{
		pUserSettings->cameraSource = Osp::Media::CAMERA_PRIMARY;
	}
	else if(pCamSrcList->IsItemChecked(1))// FRONT camera selected
	{
		pUserSettings->cameraSource = Osp::Media::CAMERA_SECONDARY;
	}
	// mainCameraResolutionIdx
	pUserSettings->mainCameraResolutionIdx = mainCamResSelIdx;
	// mainCameraFPSIdx
	pUserSettings->mainCameraFPSIdx = mainCamFpsSelIdx;
	// frontCameraResolutionIdx
	pUserSettings->frontCameraResolutionIdx = frontCamResSelIdx;
	// frontCameraFPSIdx
	pUserSettings->frontCameraFPSIdx = frontCamFpsSelIdx;
	// serverAddr
	srvAddrStr.Trim();
	pUserSettings->serverAddr = srvAddrStr;
	// serverPort
	pUserSettings->serverPort = srvPort;
}

void SettingsForm::OnActionPerformed(const Control& source, int actionId)
{
	switch(actionId)
	{
	// Soft keys:
	case ID_RIGHT_SOFTKEY:
		{
		GetSettings();
		pSmartCamApp->SaveSettings(pUserSettings);
		pSmartCamApp->DisplayMainMenuForm();
		}
		break;
	case ID_BTN_OK_CAM_SRC:
		{
		HidePopup(pCamSrcDlg);
		if(pCamSrcList->IsItemChecked(0)) // MAIN camera selected
		{
			pItemCameraSource->SetElement(ID_FORMAT_STRING, GS(L"IDS_SOURCE:") + "  " + GS(L"IDS_MAIN_CAMERA"));
			pGroupedList->RefreshItem(0, 0);
			PopulateWithMainCameraRes();
			PopulateWithMainCameraFps();
		}
		else if(pCamSrcList->IsItemChecked(1)) // FRONT camera selected
		{
			pItemCameraSource->SetElement(ID_FORMAT_STRING, GS(L"IDS_SOURCE:") + "  " + GS(L"IDS_FRONT_CAMERA"));
			pGroupedList->RefreshItem(0, 0);
			PopulateWithFrontCameraRes();
			PopulateWithFrontCameraFps();
		}
		}
		break;
	case ID_BTN_OK_CAM_RES:
		{
		HidePopup(pCamResDlg);
		Dimension* pCrtPreviewResolution = NULL;
		if(pCamSrcList->IsItemChecked(0)) // MAIN camera selected
		{
			pCrtPreviewResolution = (Dimension*)pSmartCamApp->GetMainPreviewResolutionList()->GetAt(mainCamResSelIdx);
		}
		else if(pCamSrcList->IsItemChecked(1)) // FRONT camera selected
		{
			pCrtPreviewResolution = (Dimension*)pSmartCamApp->GetFrontPreviewResolutionList()->GetAt(frontCamResSelIdx);
		}
		String crtResolutionStr = GS(L"IDS_RESOLUTION:") + "  " + Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
		pItemCameraResolution->SetElement(ID_FORMAT_STRING, crtResolutionStr);
		pGroupedList->RefreshItem(0, 1);
		}
		break;
	case ID_BTN_OK_CAM_FPS:
		{
		HidePopup(pCamFpsDlg);
		int crtFPS = 0;
		String crtFPSStr = "";
		if(pCamSrcList->IsItemChecked(0)) // MAIN camera selected
		{
			pSmartCamApp->GetMainPreviewFPSList()->GetAt(mainCamFpsSelIdx, crtFPS);
		}
		else if(pCamSrcList->IsItemChecked(1)) // FRONT camera selected
		{
			pSmartCamApp->GetFrontPreviewFPSList()->GetAt(frontCamFpsSelIdx, crtFPS);
		}
		crtFPSStr = GS(L"IDS_FRAME_RATE:") + "  " + Integer::ToString(crtFPS) + String(" fps");
		pItemCameraFPS->SetElement(ID_FORMAT_STRING, crtFPSStr);
		pGroupedList->RefreshItem(0, 2);
		}
		break;
	case ID_BTN_OK_SRV_ADDR:
		{
		HidePopup(pSrvAddrDlg);
		srvAddrStr = pSrvAddrEditField->GetText();
		srvAddrStr.Trim();
		pItemConnectionServer->SetElement(ID_FORMAT_STRING, GS(L"IDS_SERVER_ADDRESS:") + "  " + srvAddrStr);
		pGroupedList->RefreshItem(1, 0);
		}
		break;
	case ID_BTN_OK_SRV_PORT:
		{
		HidePopup(pSrvPortDlg);
		if(IsFailed(Integer::Parse(pSrvPortEditField->GetText(), srvPort)))
			srvPort = 9361;
		String srvPortStr = GS(L"IDS_PORT:") + "  " + Integer::ToString(srvPort);
		pItemConnectionPort->SetElement(ID_FORMAT_STRING, srvPortStr);
		pGroupedList->RefreshItem(1, 1);
		}
		break;
	default:
		break;
	}
}

void SettingsForm::OnItemStateChanged(const Osp::Ui::Control &source, int groupIndex, int itemIndex, int itemId, Osp::Ui::ItemStatus status)
{
	if(status == ITEM_SELECTED)
	{
		switch (itemId)
		{
		case ID_LISTITEM_CAM_SRC:
			AppLog("SettingsForm::OnItemStateChanged() - 1: camera source selected");
			ShowPopup(pCamSrcDlg);
			break;
		case ID_LISTITEM_CAM_RES:
			AppLog("SettingsForm::OnItemStateChanged() - 1: camera resolution selected");
			ShowPopup(pCamResDlg);
			break;
		case ID_LISTITEM_CAM_FPS:
			AppLog("SettingsForm::OnItemStateChanged() - 1: camera FPS selected");
			ShowPopup(pCamFpsDlg);
			break;
		case ID_LISTITEM_CONN_ADDR:
			AppLog("SettingsForm::OnItemStateChanged() - 1: connection server selected");
			srvAddrStr.Trim();
			pSrvAddrEditField->SetText(srvAddrStr);
			ShowPopup(pSrvAddrDlg);
			break;
		case ID_LISTITEM_CONN_PORT:
			AppLog("SettingsForm::OnItemStateChanged() - 1: connection port selected");
			pSrvPortEditField->SetText(Integer::ToString(srvPort));
			ShowPopup(pSrvPortDlg);
			break;
		default:
			break;
		}
	}
}

void SettingsForm::OnItemStateChanged(const Osp::Ui::Control &source, int index, int itemId, Osp::Ui::ItemStatus status)
{
	if(status == ITEM_CHECKED)
	{
		if(&source == pCamResList)
		{
			if(pCamSrcList->IsItemChecked(0))// MAIN camera selected
			{
				mainCamResSelIdx = index;
			}
			else if(pCamSrcList->IsItemChecked(1))// FRONT camera selected
			{
				frontCamResSelIdx = index;
			}
		}
		else if(&source == pCamFpsList)
		{
			if(pCamSrcList->IsItemChecked(0))// MAIN camera selected
			{
				mainCamFpsSelIdx = index;
			}
			else if(pCamSrcList->IsItemChecked(1))// FRONT camera selected
			{
				frontCamFpsSelIdx = index;
			}
		}
	}
}

void SettingsForm::OnItemStateChanged(const Osp::Ui::Control &source, int groupIndex, int itemIndex, int itemId, int elementId, Osp::Ui::ItemStatus status)
{
	if(status == ITEM_SELECTED)
	{
		switch (itemId)
		{
		case ID_LISTITEM_CAM_SRC:
			AppLog("SettingsForm::OnItemStateChanged() - 2: camera source selected");
			ShowPopup(pCamSrcDlg);
			break;
		case ID_LISTITEM_CAM_RES:
			AppLog("SettingsForm::OnItemStateChanged() - 2: camera resolution selected");
			ShowPopup(pCamResDlg);
			break;
		case ID_LISTITEM_CAM_FPS:
			AppLog("SettingsForm::OnItemStateChanged() - 2: camera FPS selected");
			ShowPopup(pCamFpsDlg);
			break;
		case ID_LISTITEM_CONN_ADDR:
			AppLog("SettingsForm::OnItemStateChanged() - 2: connection server selected");
			srvAddrStr.Trim();
			pSrvAddrEditField->SetText(srvAddrStr);
			ShowPopup(pSrvAddrDlg);
			break;
		case ID_LISTITEM_CONN_PORT:
			AppLog("SettingsForm::OnItemStateChanged() - 2: connection port selected");
			pSrvPortEditField->SetText(Integer::ToString(srvPort));
			ShowPopup(pSrvPortDlg);
			break;
		default:
			break;
		}
	}
}

void SettingsForm::OnOrientationChanged(const Control &source, OrientationStatus orientationStatus)
{
	AppLog("SettingsForm::OnOrientationChanged(): orientation = %d", GetOrientationStatus());
	DoLayout();
	RequestRedraw();
}

void SettingsForm::DoLayout()
{
	ComputeLayouts();

	CustomListItemFormat* pNewItemFormat = new CustomListItemFormat();
	pNewItemFormat->Construct();

	Rectangle clientRect = GetClientAreaBounds();
	int strItemElWidth = clientRect.width - listItemImgSize - 2 * listItemImgHorizMargin;
	int strItemElX = clientRect.width - strItemElWidth;
	int strItemElY = (listItemHeight - listItemTextSize)/2;
	pNewItemFormat->AddElement(ID_FORMAT_STRING, Rectangle(strItemElX, strItemElY, strItemElWidth, listItemTextSize), listItemTextSize);
	pNewItemFormat->AddElement(ID_FORMAT_BITMAP, Rectangle(listItemImgHorizMargin, (listItemHeight - listItemImgSize)/2, listItemImgSize, listItemImgSize));
	pNewItemFormat->SetElementEventEnabled(ID_FORMAT_STRING, true);
	pNewItemFormat->SetElementEventEnabled(ID_FORMAT_BITMAP, true);

	if(pItemFormat != NULL)
	{
		delete pItemFormat;
	}
	pItemFormat = pNewItemFormat;

	if(pItemCameraSource != NULL)
	{
		pItemCameraSource->SetItemFormat(*pItemFormat);
	}
	if(pItemCameraResolution != NULL)
	{
		pItemCameraResolution->SetItemFormat(*pItemFormat);
	}
	if(pItemCameraFPS != NULL)
	{
		pItemCameraFPS->SetItemFormat(*pItemFormat);
	}
	if(pItemConnectionServer != NULL)
	{
		pItemConnectionServer->SetItemFormat(*pItemFormat);
	}
	if(pItemConnectionPort != NULL)
	{
		pItemConnectionPort->SetItemFormat(*pItemFormat);
	}

	if(pGroupedList != NULL)
	{
		pGroupedList->SetBounds(0, 0, groupedListWidth, groupedListHeight);
	}
}

void SettingsForm::ComputeLayouts()
{
	int x,y;
	OrientationStatus orientationStatus = GetOrientationStatus();
	orientationStatus = ORIENTATION_STATUS_LANDSCAPE;

	AppLog("SettingsForm::ComputeLayouts(): orientation = %d", orientationStatus);

	GetBounds(x,y,groupedListWidth, groupedListHeight);

	if (orientationStatus == ORIENTATION_STATUS_LANDSCAPE ||
		orientationStatus == ORIENTATION_STATUS_LANDSCAPE_REVERSE)
	{
		listItemHeight = LIST_ITEM_HEIGHT_LANDSCAPE;
		listItemTextSize = LIST_ITEM_TEXT_SIZE_LANDSCAPE;
		listItemImgSize = LIST_ITEM_IMG_SIZE_LANDSCAPE;
		listItemImgHorizMargin = LIST_ITEM_IMG_HORIZ_MARGIN_LANDSCAPE;
		groupedListHeight = GROUPED_LIST_HEIGHT_LANDSCAPE;
		groupedListWidth = GROUPED_LIST_WIDTH_LANDSCAPE;
	}
	else // portrait and other, like orientation none
	{
		listItemHeight = LIST_ITEM_HEIGHT_PORTRAIT;
		listItemTextSize = LIST_ITEM_TEXT_SIZE_PORTRAIT;
		listItemImgSize = LIST_ITEM_IMG_SIZE_PORTRAIT;
		listItemImgHorizMargin = LIST_ITEM_IMG_HORIZ_MARGIN_PORTRAIT;
		groupedListHeight = GROUPED_LIST_HEIGHT_PORTRAIT;
		groupedListWidth = GROUPED_LIST_WIDTH_PORTRAIT;
	}
}

void SettingsForm::CreateCamSrcDlg()
{
	AppLog("SettingsForm::CreateCamSrcDlg()");
	pCamSrcDlg = new Popup();
	pCamSrcDlg->Construct(L"IDP_CAM_SRC_DLG");
	pCamSrcList = static_cast<List*>(pCamSrcDlg->GetControl(L"IDC_LIST_CAM_SRC"));
	pCamSrcList->RemoveAllItems();
	String mainCameraItem = GS(L"IDS_MAIN_CAMERA");
	String fronCameraItem = GS(L"IDS_FRONT_CAMERA");
    // Add a item to the List
	pCamSrcList->AddItem(&mainCameraItem, NULL, NULL, NULL, ID_LISTITEM_CAM_SRC_MAIN);
	pCamSrcList->AddItem(&fronCameraItem, NULL, NULL, NULL, ID_LISTITEM_CAM_SRC_FRONT);

	pCamSrcOkBtn = static_cast<Button*>(pCamSrcDlg->GetControl(L"IDC_BTN_OK_CAM_SRC"));
	pCamSrcOkBtn->SetActionId(ID_BTN_OK_CAM_SRC);
	pCamSrcOkBtn->AddActionEventListener(*this);
}

void SettingsForm::PopulateWithMainCameraRes()
{
	// Resolution list
	pCamResList->RemoveAllItems();
	Osp::Base::Collection::IList* pMainPreviewResolutionList = pSmartCamApp->GetMainPreviewResolutionList();
	if(pMainPreviewResolutionList != NULL)
	{
		for(int i = 0; i < pMainPreviewResolutionList->GetCount(); i++)
		{
			Dimension* pCrtPreviewResolution = (Dimension*)pMainPreviewResolutionList->GetAt(i);
			String crtResolutionStr = Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
			pCamResList->AddItem(&crtResolutionStr, NULL, NULL, NULL);
		}
	}
	pCamResList->SetItemChecked(mainCamResSelIdx, true);
	Dimension* pCrtPreviewResolution = (Dimension*)pMainPreviewResolutionList->GetAt(mainCamResSelIdx);
	String crtResolutionStr = GS(L"IDS_RESOLUTION:") + "  " + Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
	pItemCameraResolution->SetElement(ID_FORMAT_STRING, crtResolutionStr);
	pGroupedList->RefreshItem(0, 1);
}

void SettingsForm::PopulateWithMainCameraFps()
{
	// FPS list
	pCamFpsList->RemoveAllItems();
	Osp::Base::Collection::IListT<int>* pMainPreviewFPSList = pSmartCamApp->GetMainPreviewFPSList();
	if(pMainPreviewFPSList != NULL)
	{
		for(int i = 0; i < pMainPreviewFPSList->GetCount(); i++)
		{
			int crtFPS = 0;
			pMainPreviewFPSList->GetAt(i, crtFPS);
			String crtFPSStr = Integer::ToString(crtFPS);
			pCamFpsList->AddItem(&crtFPSStr, NULL, NULL, NULL);
		}
	}
	pCamFpsList->SetItemChecked(mainCamFpsSelIdx, true);
	int crtFPS = 0;
	pMainPreviewFPSList->GetAt(mainCamFpsSelIdx, crtFPS);
	String crtFPSStr = GS(L"IDS_FRAME_RATE:") + "  " + Integer::ToString(crtFPS) + String(" fps");
	pItemCameraFPS->SetElement(ID_FORMAT_STRING, crtFPSStr);
	pGroupedList->RefreshItem(0, 2);
}

void SettingsForm::PopulateWithFrontCameraRes()
{
	// Resolution list
	pCamResList->RemoveAllItems();
	Osp::Base::Collection::IList* pFrontPreviewResolutionList = pSmartCamApp->GetFrontPreviewResolutionList();
	if(pFrontPreviewResolutionList != NULL)
	{
		for(int i = 0; i < pFrontPreviewResolutionList->GetCount(); i++)
		{
			Dimension* pCrtPreviewResolution = (Dimension*)pFrontPreviewResolutionList->GetAt(i);
			String crtResolutionStr = Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
			pCamResList->AddItem(&crtResolutionStr, NULL, NULL, NULL);
		}
	}
	pCamResList->SetItemChecked(frontCamResSelIdx, true);
	Dimension* pCrtPreviewResolution = (Dimension*)pFrontPreviewResolutionList->GetAt(frontCamResSelIdx);
	String crtResolutionStr = GS(L"IDS_RESOLUTION:") + "  " + Integer::ToString(pCrtPreviewResolution->width) + " x " + Integer::ToString(pCrtPreviewResolution->height);
	pItemCameraResolution->SetElement(ID_FORMAT_STRING, crtResolutionStr);
	pGroupedList->RefreshItem(0, 1);
}

void SettingsForm::PopulateWithFrontCameraFps()
{
	// FPS list
	pCamFpsList->RemoveAllItems();
	Osp::Base::Collection::IListT<int>* pFrontPreviewFPSList = pSmartCamApp->GetFrontPreviewFPSList();
	if(pFrontPreviewFPSList != NULL)
	{
		for(int i = 0; i < pFrontPreviewFPSList->GetCount(); i++)
		{
			int crtFPS = 0;
			pFrontPreviewFPSList->GetAt(i, crtFPS);
			String crtFPSStr = Integer::ToString(crtFPS);
			pCamFpsList->AddItem(&crtFPSStr, NULL, NULL, NULL);
		}
	}
	pCamFpsList->SetItemChecked(frontCamFpsSelIdx, true);
	int crtFPS = 0;
	pFrontPreviewFPSList->GetAt(frontCamFpsSelIdx, crtFPS);
	String crtFPSStr = GS(L"IDS_FRAME_RATE:") + "  " + Integer::ToString(crtFPS) + String(" fps");
	pItemCameraFPS->SetElement(ID_FORMAT_STRING, crtFPSStr);
	pGroupedList->RefreshItem(0, 2);
}

void SettingsForm::CreateCamResDlg()
{
	AppLog("SettingsForm::CreateCamResDlg()");
	pCamResDlg = new Popup();
	pCamResDlg->Construct(L"IDP_CAM_RES_DLG");
	pCamResList = static_cast<List*>(pCamResDlg->GetControl(L"IDC_LIST_CAM_RES"));
	pCamResList->AddItemEventListener(*this);
	pCamResList->RemoveAllItems();
	if(pCamSrcList->IsItemChecked(0))// MAIN camera selected
	{
		PopulateWithMainCameraRes();
	}
	else if(pCamSrcList->IsItemChecked(1))// FRONT camera selected
	{
		PopulateWithFrontCameraRes();
	}
	pCamResOkBtn = static_cast<Button*>(pCamResDlg->GetControl(L"IDC_BTN_OK_CAM_RES"));
	pCamResOkBtn->SetActionId(ID_BTN_OK_CAM_RES);
	pCamResOkBtn->AddActionEventListener(*this);
}

void SettingsForm::CreateCamFPSDlg()
{
	AppLog("SettingsForm::CreateCamFPSDlg()");
	pCamFpsDlg = new Popup();
	pCamFpsDlg->Construct(L"IDP_CAM_FPS_DLG");
	pCamFpsList = static_cast<List*>(pCamFpsDlg->GetControl(L"IDC_LIST_CAM_FPS"));
	pCamFpsList->AddItemEventListener(*this);
	pCamFpsList->RemoveAllItems();
	if(pCamSrcList->IsItemChecked(0))// MAIN camera selected
	{
		PopulateWithMainCameraFps();
	}
	else if(pCamSrcList->IsItemChecked(1))// FRONT camera selected
	{
		PopulateWithFrontCameraFps();
	}
	pCamFpsOkBtn = static_cast<Button*>(pCamFpsDlg->GetControl(L"IDC_BTN_OK_CAM_FPS"));
	pCamFpsOkBtn->SetActionId(ID_BTN_OK_CAM_FPS);
	pCamFpsOkBtn->AddActionEventListener(*this);
}

void SettingsForm::CreateSrvAddrDlg()
{
	AppLog("SettingsForm::CreateSrvAddrDlg()");
	pSrvAddrDlg = new Popup();
	pSrvAddrDlg->Construct(L"IDP_SRV_ADDR_DLG");
	Osp::Ui::Controls::Label* pLabel = static_cast<Osp::Ui::Controls::Label*>(pSrvAddrDlg->GetControl(L"IDC_LABEL"));
	pLabel->SetBackgroundColor(Osp::Ui::Controls::SYSTEM_COLOR_POPUP_BACKGROUND);
	pSrvAddrEditField = static_cast<EditField*>(pSrvAddrDlg->GetControl(L"IDC_EDITFIELD_SRV_ADDR"));
	pSrvAddrOkBtn = static_cast<Button*>(pSrvAddrDlg->GetControl(L"IDC_BTN_OK_SRV_ADDR"));
	pSrvAddrOkBtn->SetActionId(ID_BTN_OK_SRV_ADDR);
	pSrvAddrOkBtn->AddActionEventListener(*this);
}

void SettingsForm::CreateSrvPortDlg()
{
	AppLog("SettingsForm::CreateSrvPortDlg()");
	pSrvPortDlg = new Popup();
	pSrvPortDlg->Construct(L"IDP_SRV_PORT_DLG");
	Osp::Ui::Controls::Label* pLabel = static_cast<Osp::Ui::Controls::Label*>(pSrvPortDlg->GetControl(L"IDC_LABEL"));
	pLabel->SetBackgroundColor(Osp::Ui::Controls::SYSTEM_COLOR_POPUP_BACKGROUND);
	pSrvPortEditField = static_cast<EditField*>(pSrvPortDlg->GetControl(L"IDC_EDITFIELD_SRV_PORT"));
	pSrvPortEditField->SetCurrentInputModeCategory(EDIT_INPUTMODE_NUMERIC);
	pSrvPortOkBtn = static_cast<Button*>(pSrvPortDlg->GetControl(L"IDC_BTN_OK_SRV_PORT"));
	pSrvPortOkBtn->SetActionId(ID_BTN_OK_SRV_PORT);
	pSrvPortOkBtn->AddActionEventListener(*this);
}

void SettingsForm::ShowPopup(Popup* pPopup)
{
    pPopup->SetShowState(true);
    pPopup->Show();
}

void SettingsForm::HidePopup(Popup* pPopup)
{
    pPopup->SetShowState(false);
    Draw();
    Show();
}

int SettingsForm::GetSelectedIndex(Osp::Ui::Controls::List* pList)
{
	int selIdx = -1;
	for(int i = 0; i < pList->GetItemCount(); i++)
	{
		if(pList->IsItemChecked(i))
		{
			selIdx = i;
			break;
		}
	}
	return selIdx;
}

String SettingsForm::GS(String resId)
{
	String trans;
	SmartCam *pApp = static_cast<SmartCam*>(Osp::App::Application::GetInstance());
	pApp->GetAppResource()->GetString(resId, trans);
	if (trans.IsEmpty()) trans = resId;
	return trans;
}

