/*
 * Copyright (C) 2011 Filippo Forchino <filippo.forchino@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// JpegHandler.cpp

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "video/JpegHandler.h"
#include "SmartCam.h"

#define JPEG_QUALITY 70
#define PKG_HEADER_SIZE 4
#define MAX_ENCODEQ_SIZE 1

JpegHandler::JpegHandler():
	planeHeight(0),
	yPlane(NULL),
	uPlane(NULL),
	vPlane(NULL),
	cinfoCreated(FALSE),
	headerAndWriteByteBuffer(NULL),
	headerAndWriteBuffer(NULL),
	writeBuffer(NULL),
	writeBufferSize(DEFAULT_JPEG_WRITE_BUFFER_SIZE),
	written(0)
{
	headerAndWriteByteBuffer = new Osp::Base::ByteBuffer();
	headerAndWriteByteBuffer->Construct(PKG_HEADER_SIZE + writeBufferSize);
	headerAndWriteBuffer = (unsigned char*) headerAndWriteByteBuffer->GetPointer();
	writeBuffer = headerAndWriteBuffer + PKG_HEADER_SIZE;

	destMgr.init_destination = init_destination;
	destMgr.empty_output_buffer = empty_output_buffer;
	destMgr.term_destination = term_destination;

	cinfo.client_data = (void*) this;

	cinfo.err = jpeg_std_error(&jerr);
	jerr.error_exit = error_exit;
	jerr.output_message = output_message;

	StartEncodeThread();
}


JpegHandler::~JpegHandler()
{
	StopEncodeThread();

	if(cinfoCreated)
		jpeg_destroy_compress(&cinfo);
	delete headerAndWriteByteBuffer;
	headerAndWriteByteBuffer = NULL;
	headerAndWriteBuffer = NULL;
	writeBuffer = NULL;
	writeBufferSize = 0;
	written = 0;
	if(yPlane != NULL)
	{
		free(yPlane);
		yPlane = NULL;
	}
	if(uPlane != NULL)
	{
		free(uPlane);
		uPlane = NULL;
	}
	if(vPlane != NULL)
	{
		free(vPlane);
		vPlane = NULL;
	}
}

void JpegHandler::StartEncodeThread()
{
	isEncodeThreadAlive = TRUE;
/*	DWORD threadId = 0;
	hEncodeThread = CreateThread(
                        NULL,					// default security attributes
                        0,						// use default stack size
                        EncodeThreadProc,		// thread function
                        this,					// argument to thread function
                        0,						// use default creation flags
                        &threadId);				// returns the thread identifier
	if(hEncodeThread == NULL)
    {
        CUIHandler::Msg(TEXT("Could not start encode thread: (%s)"), Util::GetLastErrorMessage());
    }*/
}

void JpegHandler::StopEncodeThread()
{
/*	if(hEncodeThread == NULL)
	{
		return;
	}*/
	isEncodeThreadAlive = false;
/*    DWORD waitResult = WaitForSingleObject(hEncodeThread, 600);
	if(waitResult == WAIT_TIMEOUT)
	{
		TerminateThread(hEncodeThread, 0);
	}
    CloseHandle(hEncodeThread);
    hEncodeThread = NULL;

	list<CFrameBuffer*>::iterator qIter;
	for(qIter = encodeQ.begin(); qIter != encodeQ.end(); qIter++)
	{
		CFrameBuffer* crtBuf = *qIter;
		delete crtBuf;
	}
	encodeQ.clear();

	DeleteCriticalSection(&csQ);

	CloseHandle(hEventQNotEmpty);
	hEventQNotEmpty = NULL;*/
}

/*void JpegHandler::EncodeFrame(CFrameBuffer* frame)
{
	int qSize = 0;
	EnterCriticalSection(&csQ);
	qSize = encodeQ.size();
	if(qSize == MAX_ENCODEQ_SIZE)
	{
		CFrameBuffer* firstBuf = encodeQ.front();
		encodeQ.pop_front();
		encodeQ.push_back(frame);
		LeaveCriticalSection(&csQ);
		delete firstBuf;
	}
	else if(qSize == 0)
	{
		encodeQ.push_back(frame);
		LeaveCriticalSection(&csQ);
		SetEvent(hEventQNotEmpty);
	}
	else
	{
		encodeQ.push_back(frame);
		LeaveCriticalSection(&csQ);
	}
}

DWORD WINAPI JpegHandler::EncodeThreadProc(LPVOID lpParam)
{
	CFrameBuffer* crtBuf = NULL;
	int qSize = 0;
	JpegHandler* pJpegHandler = (JpegHandler*) lpParam;
	while(pJpegHandler->isEncodeThreadAlive)
	{
		EnterCriticalSection(&pJpegHandler->csQ);
		qSize = pJpegHandler->encodeQ.size();
		LeaveCriticalSection(&pJpegHandler->csQ);

		while(qSize == 0)
		{
			WaitForSingleObject(pJpegHandler->hEventQNotEmpty, INFINITE);
			EnterCriticalSection(&pJpegHandler->csQ);
			qSize = pJpegHandler->encodeQ.size();
			LeaveCriticalSection(&pJpegHandler->csQ);
		}
		crtBuf = pJpegHandler->encodeQ.front();
		pJpegHandler->encodeQ.pop_front();

		if(crtBuf->type == CamMediaType::MediaTypeYV12)
		{
			pJpegHandler->encodeYV12(crtBuf->data, crtBuf->width, crtBuf->height);
		}
		else if(crtBuf->type == CamMediaType::MediaTypeRGB565)
		{
			pJpegHandler->encodeRGB565(crtBuf->data, crtBuf->width, crtBuf->height, crtBuf->isReversed);
		}
		unsigned char* headerAndJpegBuffer = pJpegHandler->GetOutputBuffer();
		int headerAndJpegBufferLen = pJpegHandler->GetOutputBufferLen();
		int jpegBufferLen = headerAndJpegBufferLen - 4;
		headerAndJpegBuffer[0] = (unsigned char) 1;// jpeg data
		headerAndJpegBuffer[1] = (unsigned char) ((jpegBufferLen >> 16) & 0xFF);
		headerAndJpegBuffer[2] = (unsigned char) ((jpegBufferLen >> 8) & 0xFF);
		headerAndJpegBuffer[3] = (unsigned char) ((jpegBufferLen >> 0) & 0xFF);
		g_pEngine->SendData(headerAndJpegBuffer, headerAndJpegBufferLen);

		delete crtBuf;
	}
	return 0;
}*/

void JpegHandler::InitRGB()
{
	if(cinfoCreated)
		return;
	cinfoCreated = TRUE;
	jpeg_create_compress(&cinfo);

	cinfo.dest = &destMgr;

	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_RGB;//JCS_YCbCr;

	jpeg_set_defaults(&cinfo);

	cinfo.dct_method = JDCT_FASTEST;
	cinfo.optimize_coding = FALSE;

	SetQuality(JPEG_QUALITY);
}

void JpegHandler::InitYUV()
{
	if(cinfoCreated)
		return;
	cinfoCreated = TRUE;
	jpeg_create_compress(&cinfo);

	cinfo.dest = &destMgr;

	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_YCbCr;

	jpeg_set_defaults(&cinfo);

	cinfo.dct_method = JDCT_FASTEST;
	cinfo.optimize_coding = FALSE;

	cinfo.raw_data_in = TRUE;
	jpeg_set_colorspace(&cinfo, JCS_YCbCr);

	SetQuality(JPEG_QUALITY);

	// finished initializing, set the header bytes
	headerAndWriteBuffer[0] = (unsigned char) 0;// jpeg header
	headerAndWriteBuffer[1] = (unsigned char) ((written >> 16) & 0xFF);
	headerAndWriteBuffer[2] = (unsigned char) ((written >> 8) & 0xFF);
	headerAndWriteBuffer[3] = (unsigned char) ((written >> 0) & 0xFF);
	// set up the counters in the ByteBuffer
	headerAndWriteByteBuffer->SetPosition(0);
	headerAndWriteByteBuffer->SetLimit(PKG_HEADER_SIZE + written);

}

int JpegHandler::SetQuality(int quality)
{
/*	if(setjmp(returnPoint))
	{
		return -1;
	}*/

	jpeg_set_quality(&cinfo, quality, FALSE);
	jpeg_suppress_tables(&cinfo, FALSE);
	jpeg_write_tables(&cinfo);
	return 0;
}
int JpegHandler::EncodeYUV420SP(const unsigned char* srcBuff, int width, int height)
{
	return 0;
}

int JpegHandler::EncodeYUV420P(const Osp::Base::ByteBuffer& srcByteBuff, int width, int height)
{
	const unsigned char* srcBuff = (unsigned char*)srcByteBuff.GetPointer();
	if(planeHeight < height || yPlane == NULL)
	{
		free(yPlane);
		free(uPlane);
		free(vPlane);
		yPlane = (JSAMPARRAY) malloc(sizeof(JSAMPROW)*height);
		uPlane = (JSAMPARRAY) malloc(sizeof(JSAMPROW)*(height/2));
		vPlane = (JSAMPARRAY) malloc(sizeof(JSAMPROW)*(height/2));
		planeHeight = height;
	}

/*	if(setjmp(returnPoint))
	{
		free(yPlane); yPlane = NULL;
		free(uPlane); uPlane = NULL;
		free(vPlane); vPlane = NULL;
		return -1;
	}*/

	cinfo.image_width = width;
	cinfo.image_height = height;

	cinfo.comp_info[0].h_samp_factor = cinfo.comp_info[0].v_samp_factor = 2;
	cinfo.comp_info[1].h_samp_factor = cinfo.comp_info[1].v_samp_factor = 1;
	cinfo.comp_info[2].h_samp_factor = cinfo.comp_info[2].v_samp_factor = 1;
	for(int i = 0; i < height; i++)
		yPlane[i] = (JSAMPROW) &srcBuff[i * width];
	for(int i = 0; i < height/2; i++) {
		uPlane[i] = (JSAMPROW) &srcBuff[width * height + i * (width/2)];
		vPlane[i] = (JSAMPROW) &srcBuff[width * height + (width/2) * (height/2) + i * (width/2)];
	}

	jpeg_start_compress(&cinfo, FALSE);

	int y = 0;
	int blockrow = 0;
	while(y < height)
	{
		JSAMPARRAY planes[3];
		planes[0] = yPlane + cinfo.comp_info[0].v_samp_factor * DCTSIZE * blockrow;
		planes[1] = uPlane + cinfo.comp_info[1].v_samp_factor * DCTSIZE * blockrow;
		planes[2] = vPlane + cinfo.comp_info[2].v_samp_factor * DCTSIZE * blockrow;
		jpeg_write_raw_data(&cinfo, planes, cinfo.comp_info[0].v_samp_factor * DCTSIZE);
		y += cinfo.comp_info[0].v_samp_factor * DCTSIZE;
		blockrow++;
	}

	jpeg_finish_compress(&cinfo);

	// finished encoding, set the header bytes
	headerAndWriteBuffer[0] = (unsigned char) 1;// jpeg data
	headerAndWriteBuffer[1] = (unsigned char) ((written >> 16) & 0xFF);
	headerAndWriteBuffer[2] = (unsigned char) ((written >> 8) & 0xFF);
	headerAndWriteBuffer[3] = (unsigned char) ((written >> 0) & 0xFF);
	// set up the counters in the ByteBuffer
	headerAndWriteByteBuffer->SetPosition(0);
	headerAndWriteByteBuffer->SetLimit(PKG_HEADER_SIZE + written);

	return 0;
}

int JpegHandler::EncodeYV12(const unsigned char* srcBuff, int width, int height)
{
	if(planeHeight < height || yPlane == NULL)
	{
		free(yPlane);
		free(uPlane);
		free(vPlane);
		yPlane = (JSAMPARRAY) malloc(sizeof(JSAMPROW) * height);
		uPlane = (JSAMPARRAY) malloc(sizeof(JSAMPROW) * (height/2));
		vPlane = (JSAMPARRAY) malloc(sizeof(JSAMPROW) * (height/2));
		planeHeight = height;
	}

/*	if(setjmp(returnPoint))
	{
		free(yPlane); yPlane = NULL;
		free(uPlane); uPlane = NULL;
		free(vPlane); vPlane = NULL;
		return -1;
	}*/

	cinfo.image_width = width;
	cinfo.image_height = height;

	cinfo.comp_info[0].h_samp_factor = cinfo.comp_info[0].v_samp_factor = 2;
	cinfo.comp_info[1].h_samp_factor = cinfo.comp_info[1].v_samp_factor = 1;
	cinfo.comp_info[2].h_samp_factor = cinfo.comp_info[2].v_samp_factor = 1;
	for(int i = 0; i < height; i++)
		yPlane[i] = (JSAMPROW) &srcBuff[i * width];
	for(int i = 0; i < height/2; i++) {
		vPlane[i] = (JSAMPROW) &srcBuff[width * height + i * (width/2)];
		uPlane[i] = (JSAMPROW) &srcBuff[width * height + (width/2) * (height/2) + i * (width/2)];
	}

	jpeg_start_compress(&cinfo, FALSE);

	int y = 0;
	int blockrow = 0;
	while(y < height)
	{
		JSAMPARRAY planes[3];
		planes[0] = yPlane + cinfo.comp_info[0].v_samp_factor * DCTSIZE * blockrow;
		planes[1] = uPlane + cinfo.comp_info[1].v_samp_factor * DCTSIZE * blockrow;
		planes[2] = vPlane + cinfo.comp_info[2].v_samp_factor * DCTSIZE * blockrow;
		jpeg_write_raw_data(&cinfo, planes, cinfo.comp_info[0].v_samp_factor * DCTSIZE);
		y += cinfo.comp_info[0].v_samp_factor * DCTSIZE;
		blockrow++;
	}

	jpeg_finish_compress(&cinfo);

	return 0;
}

int JpegHandler::EncodeRGB24(const unsigned char* srcBuff, int width, int height)
{
	JSAMPROW row_pointer[1];			// pointer to a single row
	int row_stride = width * 3;			// physical row width in buffer

/*	if(setjmp(returnPoint))
	{
		return -1;
	}*/

	cinfo.image_width = width;
	cinfo.image_height = height;

	jpeg_start_compress(&cinfo, FALSE);

	while(cinfo.next_scanline < cinfo.image_height)
	{
	    row_pointer[0] = (JSAMPROW) &srcBuff[cinfo.next_scanline * row_stride];
	    jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}

	jpeg_finish_compress(&cinfo);

	return 0;
}

int JpegHandler::EncodeRGB565(const unsigned char* srcBuff, int width, int height)
{
	JSAMPROW row_pointer[1];			// pointer to a single row
	int row_stride = width * 3;			// physical row width in buffer
	short pixel = 0;
	int crtRow = 0;

/*	if(setjmp(returnPoint))
	{
		return -1;
	}*/

	unsigned char* lineRGB24 = (unsigned char*) malloc(width * 3);
	cinfo.image_width = width;
	cinfo.image_height = height;

	jpeg_start_compress(&cinfo, FALSE);

	while(cinfo.next_scanline < cinfo.image_height)
	{
/*		if(isReversed)
			crtRow = cinfo.image_height - 1 - cinfo.next_scanline;
		else*/
			crtRow = cinfo.next_scanline;

		for(int w = 0; w < width; w++)
		{
			pixel = *(((short *)srcBuff) + crtRow * width + w);
			lineRGB24[w*3] = (pixel & RGB565_MASK_RED) >> 8;
			lineRGB24[w*3 + 1] = (pixel & RGB565_MASK_GREEN) >> 3;
			lineRGB24[w*3 + 2] = (pixel & RGB565_MASK_BLUE) << 3;
		}
		row_pointer[0] = (JSAMPROW) lineRGB24;
	    jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}

	jpeg_finish_compress(&cinfo);
	free(lineRGB24);

	return 0;
}

Osp::Base::ByteBuffer* JpegHandler::GetOutputBuffer()
{
	return headerAndWriteByteBuffer;
}

void JpegHandler::init_destination(j_compress_ptr cinfo)
{
	JpegHandler* data = (JpegHandler*) cinfo->client_data;
	cinfo->dest->next_output_byte = data->writeBuffer;
	cinfo->dest->free_in_buffer = data->writeBufferSize;
}

boolean JpegHandler::empty_output_buffer(j_compress_ptr cinfo)
{
	AppLog("JpegHandler::empty_output_buffer()");
	JpegHandler* data = (JpegHandler*) cinfo->client_data;
	int pos = data->writeBufferSize;
	data->writeBufferSize *= 2;
	Osp::Base::ByteBuffer* newByteBuf = new Osp::Base::ByteBuffer();
	newByteBuf->Construct(PKG_HEADER_SIZE + data->writeBufferSize);
	data->headerAndWriteByteBuffer->Clear();
	newByteBuf->CopyFrom(*data->headerAndWriteByteBuffer);
	newByteBuf->Clear();
	delete data->headerAndWriteByteBuffer;
	data->headerAndWriteByteBuffer = newByteBuf;
	data->headerAndWriteBuffer = (unsigned char*) data->headerAndWriteByteBuffer->GetPointer();
	data->writeBuffer = data->headerAndWriteBuffer + PKG_HEADER_SIZE;

	cinfo->dest->next_output_byte = data->writeBuffer + pos;
	cinfo->dest->free_in_buffer = data->writeBufferSize - pos;
	return TRUE;
}

void JpegHandler::term_destination(j_compress_ptr cinfo)
{
	JpegHandler* data = (JpegHandler*) cinfo->client_data;
	data->written = data->writeBufferSize - cinfo->dest->free_in_buffer;
}

void JpegHandler::error_exit(j_common_ptr cinfo)
{
	JpegHandler* data = (JpegHandler*) cinfo->client_data;
	cinfo->err->format_message(cinfo, data->messageBuffer);
	AppLogException("JpegHandler::error_exit(): %s", data->messageBuffer);
	//longjmp(data->returnPoint, 1);
}

void JpegHandler::output_message(j_common_ptr cinfo)
{
	char buf[JMSG_LENGTH_MAX + 1];
	cinfo->err->format_message(cinfo, buf);
	AppLog("JpegHandler::output_message(): %s", buf);
}
